// File: android/app/src/main/kotlin/com/example/dialer/MainActivity.kt
package com.example.dialer

import android.app.role.RoleManager
import android.content.BroadcastReceiver
import android.content.ComponentName
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.os.Build
import android.os.Bundle
import android.os.Handler
import android.telecom.PhoneAccount
import android.telecom.PhoneAccountHandle
import android.telecom.TelecomManager
import android.util.Log
import io.flutter.embedding.android.FlutterActivity
import io.flutter.embedding.engine.FlutterEngine
import io.flutter.plugin.common.MethodChannel

class MainActivity : FlutterActivity() {
    private val CHANNEL = "com.example.dialer/call_events"
    private lateinit var methodChannel: MethodChannel

    // Request code for the default-dialer request.
    private val DEFAULT_DIALER_REQUEST_CODE = 1001

    // BroadcastReceiver to catch incoming call events from our ConnectionService.
    private val incomingCallReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent?) {
            val phoneNumber = intent?.getStringExtra("phoneNumber") ?: "Unknown"
            notifyIncomingCall(phoneNumber)
        }
    }

    override fun configureFlutterEngine(flutterEngine: FlutterEngine) {
        super.configureFlutterEngine(flutterEngine)
        methodChannel = MethodChannel(flutterEngine.dartExecutor.binaryMessenger, CHANNEL)
        methodChannel.setMethodCallHandler { call, result ->
            when (call.method) {
                "requestDefaultDialer" -> {
                    requestDefaultDialer()
                    result.success(null)
                }
                else -> result.notImplemented()
            }
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        // Create the intent filter for incoming calls.
        val filter = IntentFilter("com.example.dialer.INCOMING_CALL")
        // Register the receiver. (For Android 13+ we must specify not exported.)
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
            registerReceiver(incomingCallReceiver, filter, Context.RECEIVER_NOT_EXPORTED)
        } else {
            registerReceiver(incomingCallReceiver, filter)
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        unregisterReceiver(incomingCallReceiver)
    }

    override fun onResume() {
        super.onResume()
        // Check if we are now the default dialer and update the UI accordingly.
        if (isDefaultDialer()) {
            registerManagedPhoneAccount()
            notifyDefaultDialerSet()
        }
    }

    private fun isDefaultDialer(): Boolean {
        return if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
            val roleManager = getSystemService(RoleManager::class.java)
            roleManager.isRoleHeld(RoleManager.ROLE_DIALER)
        } else {
            val telecomManager = getSystemService(TELECOM_SERVICE) as TelecomManager
            telecomManager.defaultDialerPackage == packageName
        }
    }

    // Request to become the default dialer.
    private fun requestDefaultDialer() {
        try {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
                // Use the RoleManager API for Android Q and above.
                val roleManager = getSystemService(RoleManager::class.java)
                if (!roleManager.isRoleHeld(RoleManager.ROLE_DIALER)) {
                    val intent = roleManager.createRequestRoleIntent(RoleManager.ROLE_DIALER)
                    startActivityForResult(intent, DEFAULT_DIALER_REQUEST_CODE)
                    Log.d("MainActivity", "Requested ROLE_DIALER via RoleManager")
                } else {
                    Log.d("MainActivity", "Already default dialer (ROLE_DIALER held)")
                    registerManagedPhoneAccount()
                    notifyDefaultDialerSet()
                }
            } else {
                // Fallback for older versions.
                val telecomManager = getSystemService(TELECOM_SERVICE) as TelecomManager
                if (telecomManager.defaultDialerPackage != packageName) {
                    val intent = Intent(TelecomManager.ACTION_CHANGE_DEFAULT_DIALER)
                    intent.putExtra(TelecomManager.EXTRA_CHANGE_DEFAULT_DIALER_PACKAGE_NAME, packageName)
                    startActivityForResult(intent, DEFAULT_DIALER_REQUEST_CODE)
                    Log.d("MainActivity", "Requested default dialer via TelecomManager")
                } else {
                    registerManagedPhoneAccount()
                    notifyDefaultDialerSet()
                }
            }
        } catch (ex: Exception) {
            Log.e("MainActivity", "Error requesting default dialer", ex)
        }
    }

    // Handle the result from the default-dialer role request.
    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == DEFAULT_DIALER_REQUEST_CODE) {
            Log.d("MainActivity", "onActivityResult received: resultCode=$resultCode")
            // Delay a short time before checking the role.
            Handler(mainLooper).postDelayed({
                if (isDefaultDialer()) {
                    Log.d("MainActivity", "Default dialer successfully set")
                    registerManagedPhoneAccount()
                    notifyDefaultDialerSet()
                } else {
                    Log.d("MainActivity", "Default dialer not set")
                }
            }, 500)
        }
    }

    // Register a managed PhoneAccount with the TelecomManager.
    private fun registerManagedPhoneAccount() {
        val telecomManager = getSystemService(TELECOM_SERVICE) as TelecomManager
        val phoneAccountHandle = PhoneAccountHandle(
            ComponentName(this, MyConnectionService::class.java),
            "MyPhoneAccountID"
        )
        val phoneAccount = PhoneAccount.builder(phoneAccountHandle, "Icing Dialer")
            .setCapabilities(PhoneAccount.CAPABILITY_CALL_PROVIDER)
            .setShortDescription("Icing Custom Dialer")
            .build()
        telecomManager.registerPhoneAccount(phoneAccount)
    }

    // Notify Flutter that we are now the default dialer.
    private fun notifyDefaultDialerSet() {
        methodChannel.invokeMethod("onDefaultDialerSet", null)
    }

    // Notify Flutter of an incoming call.
    private fun notifyIncomingCall(phoneNumber: String) {
        methodChannel.invokeMethod("onIncomingCall", phoneNumber)
    }
}
